import nodent from 'nodent';
import map from 'lodash/map';
import inspectf from 'inspect-f';
import kebabCase from 'lodash/kebabCase';
import camelCase from 'lodash/camelCase';
import upperFirst from 'lodash/upperFirst';
import flowRight from 'lodash/flowRight';
import indent from 'indent';
import prettier from 'prettier';
import marked from 'marked';
import hljs from 'highlight.js';
import Prism from 'prismjs';
import { compiler } from 'vueify';

var pascalCase = flowRight(upperFirst, camelCase);
function toJSON(obj) {
    if (typeof obj === 'function') {
        var str = inspectf(2, obj);
        if (!/\=\>/.test(str) && /^function /.test(str) === false) {
            str = 'function ' + str;
        }
        return str;
    }
    if (obj && typeof obj === 'object') {
        var arr = Object.keys(obj).map(function (key) { return ("\"" + key + "\": " + (toJSON(obj[key]))); });
        return ("{" + (arr.join(',')) + "}");
    }
    return JSON.stringify(obj);
}

var Module = require('module');
var gunzipSync = require('zlib').gunzipSync;
var join = require('path').join;
var readFileSync = require('fs').readFileSync;
var filename = join(__dirname, '../assets/prepack.min.js.gz');
var content = gunzipSync(readFileSync(filename)).toString();
var code = "\n" + content + "\nmodule.exports = Prepack\n";
var mod = new Module(filename, module.parent);
mod._compile(code, filename);
var ref = mod.exports;
var prepackSources = ref.prepackSources;
function prepack(ref) {
    var content = ref.content;
    var prefix = ref.prefix;
    var suffix = ref.suffix;
    var globalName = ref.globalName;

    var sources = [{
        filePath: '',
        sourceMapContents: '',
        fileContents: content
    }];
    var ref$1 = prepackSources(sources);
    var code = ref$1.code;
    var str = '}).call(this);';
    var start = code.lastIndexOf(str);
    var end = start + str.length;
    if (start === -1) {
        return [prefix,code,suffix].join('\n');
    }
    return [prefix,code.slice(0, start),("}).call(" + globalName + ");"),code.slice(end, -1),
        suffix].join('\n');
}

var doc2js = function (ref) {
    var name = ref.name;
    var script = ref.script;
    var style = ref.style;

    var componentName = kebabCase(name);
    var injectCSS = style && ("\nvar insert = function (css) {\n  if (typeof window === 'undefined' || typeof document === 'undefined') return;\n  var elem = document.createElement('style')\n  elem.setAttribute('type', 'text/css')\n  elem.innerHTML = css\n\n  var head = document.getElementsByTagName('head')[0]\n  head.appendChild(elem)\n  return function () {\n    head.removeChild(elem)\n  }\n}\nexports.created = function () {\n  var css = " + (JSON.stringify(style)) + "\n  this.__clean = insert(css)\n}\nexports.destroyed = function () {\n  this.__clean()\n}\n");
    var varName = pascalCase(componentName);
    var raw = "\n" + varName + " = (function (module) {\n" + script + "\nvar exports = module.exports\nexports.name = \"" + componentName + "\"\n" + (injectCSS || '') + "\nexports.install = function (Vue) {\n  Vue.component(exports.name, exports)\n}\nreturn module.exports;\n})({});\n";
    return prepack({
        content: raw,
        prefix: 'var __proxy = {};',
        suffix: ("var component = __proxy." + varName + ";\ntypeof exports === 'object' && typeof module !== 'undefined' && (module.exports = component);\ntypeof window !== void 0 && window.Vue && Vue.use(component);\n"),
        globalName: '__proxy'
    });
};

var doc2sfc = function (arg) {
    var html = arg.html;
    var style = arg.style;
    var script = arg.script;
    var components = arg.components;
    var docInfo = arg.docInfo; if ( docInfo === void 0 ) docInfo = {};
    var stylePart = style ? ("\n<style>\n" + style + "\n</style>") : '';
    var etended = prettify(("module.exports = " + (toJSON(docInfo)) + ";"));
    return ("<template>\n<article class=\"markdown-body\">\n" + html + "\n</article >\n</template>\n\n<script lang=\"buble\">\n" + script + "\n" + etended + "\nmodule.exports.components = {\n" + (indent(components, 2)) + "\n}\n</script>\n" + stylePart + "\n");
};

function prettify(code) {
    return prettier.format(code, {
        singleQuote: true,
        parser: 'babylon'
    });
}

var reStyle = /<style(|\s*[^>]+)>([\s\S]+)<\/style>/;
var reScript = /<script(|\s*[^>]+)>([\s\S]+)<\/script>/;
var reTemplate = /<template(|\s*[^>]+)>([\s\S]+)<\/template>/;
var extractSFC = function (code) {
    var styleMatch = reStyle.exec(code);
    var scriptMatch = reScript.exec(code);
    var templateMatch = reTemplate.exec(code);
    var style = styleMatch ? styleMatch[2] : '';
    var styleAttrs = styleMatch ? styleMatch[1].trim() : '';
    var script = scriptMatch ? scriptMatch[2].trim() : '';
    var scriptAttrs = scriptMatch ? scriptMatch[1].trim() : '';
    var template = templateMatch ? templateMatch[2].trim() : '';
    var templateAttrs = templateMatch ? templateMatch[1].trim() : '';
    if (template === '') {
        template = code.replace(reStyle, '').replace(reScript, '');
    }
    if (script === '') {
        script = 'module.exports = {}';
    } else {
        script = script.replace(/export\s+default/, 'module.exports =');
    }
    return {
        style: style,
        styleAttrs: styleAttrs,
        script: script,
        scriptAttrs: scriptAttrs,
        template: template,
        templateAttrs: templateAttrs,
        demoOnly: templateAttrs.indexOf('demo-only') > -1
    };
};

var Renderer = marked.Renderer;
var getRenderer = function () { return new Renderer(); };
Renderer.prototype.heading = function (text, l) {
    return ("<h" + l + ">" + text + "</h" + l + ">");
};
Renderer.prototype.link = function (href, title, text) {
    var relative = !/^(https?:)?\/\//.test(href);
    var out = "<a href=\"" + href + "\"";
    if (relative === false) {
        out += " target=\"_blank\"";
    }
    if (title) {
        out += " title=\"" + title + "\"";
    }
    out += ">" + text + "</a>";
    return out;
};

var highlightJs = function (code, lang) { return hljs.highlight(lang, code).value; };

var prism = function (code, lang) {
    if ( lang === void 0 ) lang = 'autoit';

    if (!Prism.languages[lang]) {
        try {
            require(("prismjs/components/prism-" + lang));
        } catch (e) {
            lang = 'autoit';
            require('prismjs/components/prism-autoit');
        }
    }
    return Prism.highlight(code, Prism.languages[lang]);
};
global.Prism = Prism;

var highlightMap = {
    prism: prism,
    'highlight.js': highlightJs
};
var renderer = getRenderer();
var FIX_VUE = /<span class="hljs-tag">&lt;\/</g;
var FIXTURE = '<span class="hljs-tag"><span>&lt;</span>/<';
var fix = function (code) { return code.replace(FIX_VUE, FIXTURE); };
var transform = function (source, config) {
    var id = 0;
    var demos = [];
    renderer.code = code;
    var markup = marked(source, {
        renderer: renderer
    });
    return {
        demos: demos,
        markup: markup
    };
    function code(raw, language) {
        var highlight = config.highlight;
        var fn = null;
        if (typeof highlight === 'function') {
            fn = highlight;
        } else if (highlightMap[highlight]) {
            fn = highlightMap[highlight];
        } else {
            throw new Error('Invalid highlight option!');
        }
        var lang = language === 'vue' ? 'html' : language;
        var markup = fn(raw, lang);
        if (lang !== 'html') {
            return wrapHljsCode(fix(markup), lang);
        }
        var tag = "md2vuedemo" + ((id++).toString(36));
        var ref = extractSFC(raw);
        var style = ref.style;
        var styleAttrs = ref.styleAttrs;
        var script = ref.script;
        var scriptAttrs = ref.scriptAttrs;
        var template = ref.template;
        var templateAttrs = ref.templateAttrs;
        var demoOnly = ref.demoOnly;
        templateAttrs = templateAttrs || 'lang="html"';
        scriptAttrs = scriptAttrs || 'lang="buble"';
        var vue = "<template " + templateAttrs + ">\n  <div class=\"vue-demo\">\n" + (indent(template)) + "\n  </div>\n</template>\n\n<script " + scriptAttrs + ">\n" + script + "\n</script>";
        if (style !== '') {
            styleAttrs = styleAttrs ? (styleAttrs + " ") : '';
            vue = "<style " + styleAttrs + "scoped>" + style + "</style>\n" + vue;
        }
        demos.push({
            tag: tag,
            raw: raw,
            vue: vue
        });
        var inject = config.inject;
        var injection = '';
        var sourceCode = '';
        if (!demoOnly) {
            if (typeof inject === 'function') {
                injection = inject();
            } else if (typeof inject === 'string') {
                injection = inject;
            }
            sourceCode = wrapHljsCode(fix(markup), lang);
        }
        var demoApp = "<" + tag + "></" + tag + ">";
        var klass = demoOnly ? 'vue-demo-block vue-demo-block-demo-only' : 'vue-demo-block';
        return ("\n<div class=\"" + klass + "\">\n" + demoApp + "\n" + injection + "\n" + sourceCode + "\n</div>\n");
    }
    
};
function wrapHljsCode(code, lang) {
    return ("<pre v-pre class=\"lang-" + lang + "\"><code>" + code + "</code></pre>");
}

var kEvent = 'style';
var StyleBundler = function StyleBundler() {
    this.CSSText = '';
    this.onStyleReceived = this.onStyleReceived.bind(this);
};
StyleBundler.prototype.onStyleReceived = function onStyleReceived (ref) {
        var style = ref.style;

    this.CSSText += '\n' + style;
};
StyleBundler.prototype.reset = function reset () {
    this.CSSText = '';
    if (this.source) {
        this.source.removeListener(kEvent, this.onStyleReceived);
    }
};
StyleBundler.prototype.setSource = function setSource (source) {
    this.reset();
    this.source = source;
    source.on(kEvent, this.onStyleReceived);
    return this;
};
StyleBundler.prototype.readOnce = function readOnce (fn) {
    var CSSText = this.CSSText.trim();
    if (typeof fn === 'function') {
        fn(CSSText);
    }
    this.reset();
    return CSSText;
};

StyleBundler.from = (function (source) { return new StyleBundler().setSource(source); });

compiler.applyConfig({
    extractCSS: true,
    customCompilers: {
        buble: function buble(content, cb) {
            var ref = require('buble').transform(content);
            var code = ref.code;
            var ret = code.replace(/\n{2,}/g, '\n').replace(/^\s+/, '  ').replace(/\s+$/, '');
            cb(null, ret);
        }
    }
});
var queue = [];
var currentTask = null;
compiler.compilePromise = (function (content, filePath) {
    if ( content === void 0 ) content = '';
    if ( filePath === void 0 ) filePath = '';

    var deferred = defer();
    var task = function () {
        var bundler = StyleBundler.from(compiler);
        currentTask = deferred.promise;
        compiler.compile(content, filePath, function (err, result) {
            if (err) {
                deferred.reject(err);
            } else {
                deferred.resolve({
                    script: result,
                    style: bundler.readOnce()
                });
            }
            process.nextTick(function () {
                currentTask = null;
                if (queue.length) {
                    queue.shift()();
                }
            });
        });
    };
    if (currentTask) {
        queue.push(task);
    } else {
        task();
    }
    return deferred.promise;
});
function defer() {
    var deferred = {};
    var promise = new Promise(function (resolve, reject) {
        deferred.resolve = resolve;
        deferred.reject = reject;
    });
    deferred.promise = promise;
    return deferred;
}

nodent();
var defaults = {
    target: 'vue',
    highlight: 'highlight.js'
};
var index = function (source, config) {
    if ( config === void 0 ) config = {};

    return (function ($return, $error) {
        var name, target, extend, markup, demos, demoApps, content, script, style;
        config.name = config.name || config.componentName;
        config.extend = config.extend || config.documentInfo;
        config.inject = config.inject || config.customMarkups;
        if (config.componentName) {
            console.warn('`componentName` is deprecated, use `name` instead.');
        }
        if (config.documentInfo) {
            console.warn('`documentInfo` is deprecated, use `extend` instead.');
        }
        if (config.customMarkups) {
            console.warn('`customMarkups` is deprecated, use `inject` instead.');
        }
        config = Object.assign({}, defaults, config);
        var assign;
        ((assign = config, name = assign.name, target = assign.target, extend = assign.extend));
        var assign$1;
        ((assign$1 = transform(source, config), markup = assign$1.markup, demos = assign$1.demos));
        demoApps = [];
        var $Loop_2_local;
        function $Loop_2_step() {
            var ref = $Loop_2_local();
            var demo = ref[0];
            var $iterator_demo_1 = ref[1];
            return $Loop_2.bind(this, demo, $iterator_demo_1);
        }
        
        function $Loop_2(demo, $iterator_demo_1) {
            $Loop_2_local = function () {
                return [demo,$iterator_demo_1];
            };
            if (!($iterator_demo_1[1] = $iterator_demo_1[0].next()).done && ((demo = $iterator_demo_1[1].value) || true)) {
                var vue, tag, script, style;
                var assign;
                ((assign = demo, vue = assign.vue, tag = assign.tag));
                return compileVue(vue, tag).then((function ($await_4) {
                    var assign;
                    ((assign = $await_4, script = assign.script, style = assign.style));
                    demoApps.push({
                        style: style,
                        script: wrapVueCompiled({
                            tagName: tag,
                            compiled: script
                        })
                    });
                    return $Loop_2_step;
                }).$asyncbind(this, $error), $error);
            } else 
                { return [1]; }
        }
        
        return Function.$asyncbind.trampoline(this, $Loop_2_exit, $Loop_2_step, $error, true)($Loop_2.bind(this, undefined, [demos[Symbol.iterator]()]));
        function $Loop_2_exit() {
            content = doc2sfc({
                docInfo: extend,
                html: markup,
                style: map(demoApps, 'style').join('\n'),
                script: map(demoApps, 'script').join('\n'),
                components: map(demos, function (ref) {
                    var tag = ref.tag;

                    return ("'" + tag + "': " + tag);
            }).join(',\n')
            });
            if (!target || target === 'vue') {
                return $return(content);
            }
            if (!name) {
                return $error(new Error('[Error] `name` must be specified!'));
            }
            return compileVue(content, name).then((function ($await_5) {
                var assign;
                ((assign = $await_5, script = assign.script, style = assign.style));
                return $return(doc2js({
                    script: script,
                    style: style,
                    name: name
                }));
            }).$asyncbind(this, $error), $error);
        }
        
    }).$asyncbind(this, true);
};

function compileVue(content, path) {
    return compiler.compilePromise(content, path);
}

function wrapVueCompiled(ref) {
    var tagName = ref.tagName;
    var compiled = ref.compiled;

    var code = tagName + " = (function (module) {\n" + compiled + "\nreturn module.exports\n})({});\n";
    return prepack({
        content: code,
        prefix: ("var " + tagName + " = {};"),
        suffix: (tagName + " = " + tagName + "." + tagName + ";\n"),
        globalName: tagName
    });
}

export default index;
